/**
 *******************************************************************************
 * @file    IOSCadjustment.c
 * @brief   This file provides API functions for IOSC adjustment.
 * @version V1.0.1.3
 * $Date:: 2018-07-27 15:20:46 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA MICROELECTRONICS CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <stdlib.h>
#include "IOSCadjustment.h"
#include "trm_t32a.h"
#include "trm.h"

/**
 *  @addtogroup Example
 *  @{
 */

/** 
 *  @defgroup TRMOSC TRMOSC Sample Appli
 *  @{
 */
/*------------------------------------------------------------------------------*/
/*  Macro Function                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_macro TRMOSC Private Macro
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_macro */


/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
#ifdef TRMOSC_LOSC
#define CAPTUREVAL10MHZ        (9765)          /*!< Caputer value @10MHz */
#else
#define CAPTUREVAL10MHZ        (10416)         /*!< Caputer value @10MHz */
#endif
#define FIXEDPOINT             (100)           /*!< fixed point 2 */
#define COARSETRIMSTEP         (80)            /*!< Coarse trimming step 0.8% * FIXEDPOINT */
#define FINETRIMSTEP           (10)            /*!< Fine trimming step   0.1% * FIEXDPOINT */
#define MARGINADJUST           (20)            /*!< Adjustment margin    0.2% * FIEXDPOINT */
#define COARESELIMIT           (0x0000001fUL)  /*!< Setting limit */
#define COARESELIMIT_M         (0x00000020UL)  /*!< Setting limit */
#define FINELIMIT              (0x00000007UL)  /*!< Setting limit */
#define FINELIMIT_M            (0x00000008UL)  /*!< Setting limit */
#define TRIMCOARSE_MAX         (0x00000040UL)  /*!< max value */
#define TRIMFINE_MAX           (0x0000000FUL)  /*!< max value */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_define TRMOSC Private Define
 *  @{
 */
/* no define */

/** 
 *  @}
 */ /* End of group TRMOSC_Private_define */



/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_typedef TRMOSC Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group TRMOSC_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_variables TRMOSC Private Variables
 *  @{
 */
static IOSCADJ_TypeDef IoscAdjust;
static TRM_ControlTypeDef AdjustData;
static int32_t  difference;

/**
 *  @}
 */ /* End of group TRMOSC_Private_variables */


/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Private_fuctions TRMOSC Private Fuctions
 *  @{
 */
static void MeasureWave(IOSCADJ_TypeDef *Measure);
static int32_t CalcDiff(IOSCADJ_TypeDef *Diff);
static int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust);
static int32_t GetAdjustmentValue(void);

/*--------------------------------------------------*/
/** 
  * @brief  Measurement Waveform
  * @param  Measure  :TRMOSC adjustment Structure
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
static void MeasureWave(IOSCADJ_TypeDef *Measure)
{
    StartCapture();
    Measure->MeasureVal = GetCaptureData();
    StopCapture();
}

/*--------------------------------------------------*/
/** 
  * @brief  Calculation of difference
  * @param  Diff  :TRMOSC adjustment Structure
  * @retval TRMOSC_DONE, TRMOSC_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t CalcDiff(IOSCADJ_TypeDef *Diff)
{
    uint32_t absdiff;
    int32_t retval;

    Diff->MeasureDiff = (int32_t)((100*FIXEDPOINT)-((Diff->MeasureVal * (100*FIXEDPOINT)) / CAPTUREVAL10MHZ));
/**/
    absdiff = (uint32_t)abs(Diff->MeasureDiff);
    if (absdiff <= MARGINADJUST) 
    {
        retval = TRMOSC_DONE;
    }
    else 
    {
        retval = TRMOSC_ERROR;
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Calculation of adjustment value
  * @param  Adjust  :TRMOSC adjustment Structure
  * @retval TRMOSC_RUN, TRMOSC_ERROR
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t CalcAdjVal(IOSCADJ_TypeDef *Adjust)
{
    int32_t retval;
    uint32_t coarse, fine;

    difference = Adjust->MeasureDiff + GetAdjustmentValue();
    coarse =  ((uint32_t)abs(difference) / COARSETRIMSTEP);
    fine   = (((uint32_t)abs(difference) - (coarse * COARSETRIMSTEP)) / FINETRIMSTEP);

    if (difference < 0)
    {
        if ((coarse <= COARESELIMIT_M) && (fine <= FINELIMIT_M)) 
        {
            Adjust->CoarseTrim = -(int32_t)coarse;
            Adjust->FineTrim   = -(int32_t)fine;
            retval = TRMOSC_RUN;
        }
        else 
        {
            retval = TRMOSC_ERROR;
        }
    }
    else 
    {
        if ((coarse <= COARESELIMIT) && (fine <= FINELIMIT)) 
        {
            Adjust->CoarseTrim = (int32_t)coarse;
            Adjust->FineTrim   = (int32_t)fine;
            retval = TRMOSC_RUN;
        }
        else 
        {
            retval = TRMOSC_ERROR;
        }
    }

    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Get Adjustment value
  * @param  -
  * @retval Adjustment value
  * @note   -
  */
/*--------------------------------------------------*/
static int32_t GetAdjustmentValue(void)
{
    uint32_t trimvalueC;
    uint32_t trimvalueF;
    int32_t adjvalue;

    if (GetTrimmingControlValue() == TRMOSC_CONTROL_DISABLE) 
    {
        trimvalueC = GetInitCoarseTrimValue();
        trimvalueF = GetInitFineTrimValue();
    }
    else 
    {
        trimvalueC = GetCoarseTrimValueSet();
        trimvalueF = GetFineTrimValueSet();
    }
/**/
    if (trimvalueC > COARESELIMIT) 
    {
        trimvalueC = TRIMCOARSE_MAX - trimvalueC;
        adjvalue = -COARSETRIMSTEP * (int32_t)trimvalueC;
    }
    else 
    {
        adjvalue = COARSETRIMSTEP * (int32_t)trimvalueC;
    }
/**/
    if (trimvalueF > FINELIMIT) 
    {
        trimvalueF = TRIMFINE_MAX - trimvalueF;
        adjvalue += -FINETRIMSTEP * (int32_t)trimvalueF;
    }
    else 
    {
        adjvalue += FINETRIMSTEP * (int32_t)trimvalueF;
    }

    return (adjvalue);
}

/**
 *  @}
 */ /* End of group TRMOSC_Private_fuctions */


/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup TRMOSC_Exported_functions TRMOSC Exported Functions
 *  @{
 */

/*--------------------------------------------------*/
/** 
  * @brief  Adjustment of internal OSC
  * @param  -
  * @retval TRMOSC_DONE, TRMOSC_ERROR, TRMOSC_RUN
  * @note   -
  */
/*--------------------------------------------------*/
int32_t IOSCAdjustment(void)
{
    int32_t retval;

    MeasureWave(&IoscAdjust);                                           /* Measurement Waveform */
    retval = CalcDiff(&IoscAdjust);                                     /* Calculation of difference */
    if (retval == TRMOSC_ERROR) 
    {
        retval = CalcAdjVal(&IoscAdjust);                               /* Calculation of adjustment value */
        if(retval == TRMOSC_RUN) 
        {
            AdjustData.TrimmingControl     = TRMOSC_CONTROL_ENABLE;
            AdjustData.CoarseTrimmingValue = (uint32_t)IoscAdjust.CoarseTrim;
            AdjustData.FineTrimmingValue   = (uint32_t)IoscAdjust.FineTrim;
            SetTrmOsc(&AdjustData);                                     /* Set TRMOSC register */
        }
        else 
        {                                                           /* TRMOSC_ERROR */
            /* do nothing */
        }
    }
    else 
    {                                                               /* TRMOSC_DONE  */
        /* do nothing */
    }
    return (retval);
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize TRM Function
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
void InitTRM(void)
{
    TRM_ControlTypeDef InitData;

    InitData.TrimmingControl     = TRMOSC_CONTROL_DISABLE;
    InitData.CoarseTrimmingValue = 0;
    InitData.FineTrimmingValue   = 0;
    SetTrmOsc(&InitData);
}

/**
 *  @}
 */ /* End of group TRMOSC_Exported_functions */

/**
 *  @}
 */ /* End of group TRMOSC */

/**
 *  @} 
 */ /* End of group Example */

#ifdef __cplusplus
}
#endif /* __cplusplus */
